package introspection;

import java.util.*;
import java.lang.reflect.*;

@SuppressWarnings("all")
public class ClassInspector {
	private Class cl;
	private String clName;
	
	public ClassInspector(String className) {
		clName = className;
		try {
			cl = Class.forName(clName);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	public Vector<String> getParentClasses() {
		Vector<String> parents = new Vector<String>();
		Class subClass = cl;
		Class superClass;
		parents.add(subClass.getName());
		superClass = subClass.getSuperclass();
		while (superClass != null) {
			parents.add(0, superClass.getName());
			subClass = superClass;
			superClass = subClass.getSuperclass();
		}
		return parents;
	}
	public Vector getModifiers() { 
		Vector modifiers = new Vector();
		int m = cl.getModifiers(); 
		if (Modifier.isPublic(m))   modifiers.add("public"); 
		if (Modifier.isAbstract(m))	modifiers.add("abstract"); 
		if (Modifier.isFinal(m))    modifiers.add("final"); 
		return modifiers; 
	}
	public Vector getInterfaces() {  
		Vector interfaces = new Vector(); 
		for (Class c : cl.getInterfaces()) { 
			interfaces.add(c.getName()); 
		} 
		return interfaces; 
	}
	public Vector getPublicFields() { 
		Vector pf = new Vector(); 
		for (Field f : cl.getFields()) 
			pf.add(f.getType().getName()+" "+f.getName()); 
		return pf; 
	}
	private String rechercheParametres(Class[] classes) { 
		StringBuffer param = new StringBuffer("("); 
		for (int i = 0; i < classes.length; i ++) { 
			param.append(formatParametre(classes[i].getName())); 
			if (i < classes.length - 1)  
				param.append(", "); 
		} 
		param.append(")"); 
		return param.toString(); 
	}
	private String formatParametre(String s) { 
		if (s.charAt(0) == '[') { 
			StringBuffer param = new StringBuffer(""); 
			int dimension = 0; 
			while (s.charAt(dimension) == '[') dimension++; 
			switch(s.charAt(dimension)) { 
				case 'B' : param.append("byte");break; 
				case 'C' : param.append("char");break; 
				case 'D' : param.append("double");break; 
				case 'F' : param.append("float");break; 
				case 'I' : param.append("int");break; 
				case 'J' : param.append("long");break; 
				case 'S' : param.append("short");break; 
				case 'Z' : param.append("boolean");break; 
				case 'L' : param.append(s.substring(dimension+1,s.indexOf(";"))); 
			} 
			for (int i =0; i < dimension; i++) 
				param.append("[]"); 
			return param.toString();  
		} else
			return s; 
	}
	public Vector getConstructors() {
		Vector constr = new Vector(); 
		Constructor[] constructeurs = cl.getConstructors(); 
		for (Constructor c : cl.getConstructors()) { 
			constr.add(rechercheParametres(c.getParameterTypes())); 
		}
		return constr;
	}
	public Vector getPublicMethods() {  
		Vector pm = new Vector();
		for (Method m : cl.getMethods()) {
			StringBuffer methode = new StringBuffer();
			methode.append(formatParametre(m.getReturnType().getName())); 
			methode.append(" ");
			methode.append(m.getName());
			methode.append(rechercheParametres(m.getParameterTypes())); 
			pm.add(methode.toString());
		}
		return pm;
	}
	public List getAllMethods() {
		List am = new ArrayList();
		for (Method m : cl.getDeclaredMethods()) { 
			StringBuffer methode = new StringBuffer(); 
			methode.append(formatParametre(m.getReturnType().getName())); 
			methode.append(" ");
			methode.append(m.getName());
			methode.append(rechercheParametres(m.getParameterTypes())); 
			am.add(methode.toString());
		}
		return am;
	}

}
